==> 0.0.2.tar.gz: example/lib/yf_ads_request_native_draw_feed_video_view.dart <==
import 'dart:core';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/ads/yf_ads_native_draw_feed_video_view.dart';
import 'package:yf_ads/callbacks/yf_ads_native_draw_feed_video_callback.dart';
import 'package:video_player/video_player.dart';
import 'package:yf_ads/yf_ads.dart';

import 'yf_ads_call_back_method.dart';
import 'yf_ads_draw_native_express_video_params.dart';

class YFAdsNativeDrawFeedVideoViewDemo extends StatefulWidget {
  const YFAdsNativeDrawFeedVideoViewDemo({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return YFAdsNativeDrawFeedVideoViewDemoState();
  }
}

class YFAdsNativeDrawFeedVideoViewDemoState
    extends State<YFAdsNativeDrawFeedVideoViewDemo> {
  VideoPlayerController? _controller;
  VideoPlayer? _videoPlayer;
  var videoPause = false;

  List<VideoParams> videoDatas = [];

  var initVideoIndex = 0;
  var currentPageIndex = 0;

  final totalChildViewSize = 6;
  final totalAdViewSize = 2;

  PageController? _pageController;

  var cacheAd = false;
  Map<int, bool> viewRenderStatusMap = Map();

  final videoPaths = [
    'assets/video11.mp4',
    'assets/video12.mp4',
    'assets/video13.mp4',
    'assets/video14.mp4',
    'assets/video_2.mp4'
  ];

  final videoFirstFramePaths = [
    'assets/video11.png',
    'assets/video12.png',
    'assets/video13.png',
    'assets/video14.png',
    'assets/img_video_2.png'
  ];

  @override
  void initState() {
    super.initState();
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersive);
    currentPageIndex = initVideoIndex;
    _pageController = PageController(
      initialPage: initVideoIndex,
    );
    int totalLength = videoPaths.length;
    for (int index = 0; index < totalLength; index++) {
      var videoParams = VideoParams(
          videoPath: videoPaths[index],
          videoFirstFramePath: videoFirstFramePaths[index],
          type: VideoParams.TYPE_VIDEO);
      videoDatas.add(videoParams);
    }
    for (int index = 0; index < totalAdViewSize; index++) {
      var videoParams = VideoParams(
          videoPath: null,
          videoFirstFramePath: null,
          type: VideoParams.TYPE_AD);
      videoDatas.insert(index * 2 + 2, videoParams);
    }
    _initVideoPlayerState(initVideoIndex);
  }

  _initVideoPlayerState(int index) {
    videoPause = false;
    initVideoIndex = index;
    final controller =
        VideoPlayerController.asset(videoDatas[index].videoPath!);
    final old = _controller;

    old?.pause();
    debugPrint("video contoller paused.");
    _controller = controller;
    _videoPlayer = VideoPlayer(_controller!);

    debugPrint("video contoller changed.");
    setState(() {});

    controller
      ..initialize().then((_) {
        debugPrint("video controller initialized");
        old?.dispose();
        controller.play();
        setState(() {});
      });

    controller.setLooping(true);
  }

  void videoPlayerTaped() {
    bool videoPlaying = _controller!.value.isPlaying;
    if (videoPlaying) {
      _controller!.pause();
    } else {
      _controller!.play();
    }
    print("videoPlayerTaped videoPlaying $videoPlaying");
    setState(() {
      videoPause = videoPlaying;
    });
  }

  void _pageChanged(int index) {
    print("_pageChanged index is $index");
    currentPageIndex = index;
    var videoData = videoDatas[index];
    if (videoData.type == VideoParams.TYPE_AD) {
      print("_pageChanged ad view return");
      _controller!.pause();
      if (!cacheAd) {
        setState(() {
          viewRenderStatusMap[index] = false;
        });
      }
      return;
    }
    _initVideoPlayerState(index);
  }

  void adViewCallBack(YFAdsNativeDrawFeedVideoCallBack reaperAdCallBack) {
    print(
        "adViewCallBack CallBack is ${reaperAdCallBack.method} videId is ${reaperAdCallBack.viewId}");
    if (YFAdsCallBackMethod.METHOD_DRAW_FEED_VIDEO_SUCCESS ==
        reaperAdCallBack.method) {
      int viewId = reaperAdCallBack.viewId!;
      if (!cacheAd) {
        setState(() {
          viewRenderStatusMap[viewId] = true;
        });
        return;
      }
      bool? cached = viewRenderStatusMap[viewId];
      print("adViewCallBack cached $cached");
      if (cached == null || !cached) {
        setState(() {
          viewRenderStatusMap[viewId] = true;
        });
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    var pauseView = videoPause
        ? Container(
            width: 48,
            height: 48,
            child: Icon(Icons.play_arrow),
            decoration: BoxDecoration(
                color: Colors.blue,
                borderRadius: BorderRadius.all(Radius.circular(24))),
          )
        : Container();

    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;

    var curVideoData = videoDatas[initVideoIndex];
    var overlayPictureWidget = curVideoData.type == VideoParams.TYPE_VIDEO
        ? Container(
            child: Image.asset(
            videoDatas[initVideoIndex].videoFirstFramePath!,
            fit: BoxFit.cover,
            height: double.infinity,
            width: double.infinity,
            alignment: Alignment.center,
          ))
        : Container(
            width: 0,
            height: 0,
          );

    var videoChildView = Center(
      child: _controller!.value.isInitialized
          ? AspectRatio(
              aspectRatio: _controller!.value.aspectRatio,
              child: GestureDetector(
                child: Stack(
                  alignment: AlignmentDirectional.center,
                  children: [_videoPlayer!, pauseView],
                ),
                onTap: videoPlayerTaped,
              ))
          : overlayPictureWidget,
    );

    var adProgressView = Center(
      child: Container(
        width: double.infinity,
        height: double.infinity,
        color: Colors.black12,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            CircularProgressIndicator(),
            Text(
              "加载中",
              textAlign: TextAlign.center,
              style: TextStyle(
                fontSize: 8,
                color: Colors.grey,
                decoration: TextDecoration.none,
              ),
            )
          ],
        ),
      ),
    );

    var pageView = PageView.builder(
      scrollDirection: Axis.vertical,
      itemBuilder: (BuildContext context, int index) {
        var videoData = videoDatas[index];
        if (videoData.type == VideoParams.TYPE_AD) {
          bool? viewRendered = viewRenderStatusMap[index];
          return Stack(
            alignment: AlignmentDirectional.center,
            children: [
              YFAdsNativeDrawFeedVideoView(
                backgroundColor: Colors.white70,
                cache: cacheAd,
                interruptVideoPlay: true,
                uniqueId: index,
                positionId: "a2c9d554-17d5-4a50-a5b6-c955eb5e3855",
                width: width,
                height: height,
                onAdCallBack: adViewCallBack,
              ),
              viewRendered == null || !viewRendered
                  ? adProgressView
                  : Container(
                      width: 0,
                      height: 0,
                    )
            ],
          );
        }
        return videoChildView;
      },
      onPageChanged: _pageChanged,
      controller: _pageController,
      itemCount: videoDatas.length,
    );

    return pageView;
  }

  @override
  void dispose() {
    super.dispose();
    _controller!.dispose();
    YFAds.instance.clearNativeDrawFeedVideoPageCache();
  }
}

==> 0.0.8.tar.gz: example/lib/yf_ads_request_native_draw_feed_video_view.dart <==
import 'dart:core';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/ads/yf_ads_native_draw_feed_video_view.dart';
import 'package:yf_ads/callbacks/yf_ads_native_draw_feed_video_callback.dart';
import 'package:video_player/video_player.dart';
import 'package:yf_ads/yf_ads.dart';

import 'yf_ads_call_back_method.dart';
import 'yf_ads_draw_native_express_video_params.dart';

class YFAdsNativeDrawFeedVideoViewDemo extends StatefulWidget {
  const YFAdsNativeDrawFeedVideoViewDemo({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return YFAdsNativeDrawFeedVideoViewDemoState();
  }
}

class YFAdsNativeDrawFeedVideoViewDemoState
    extends State<YFAdsNativeDrawFeedVideoViewDemo> {
  VideoPlayerController? _controller;
  VideoPlayer? _videoPlayer;
  var videoPause = false;

  List<VideoParams> videoDatas = [];

  var initVideoIndex = 0;
  var currentPageIndex = 0;

  final totalChildViewSize = 6;
  final totalAdViewSize = 2;

  PageController? _pageController;

  var cacheAd = false;
  Map<String, bool> viewRenderStatusMap = Map();

  final videoPaths = [
    'assets/video11.mp4',
    'assets/video12.mp4',
    'assets/video13.mp4',
    'assets/video14.mp4',
    'assets/video_2.mp4'
  ];

  final videoFirstFramePaths = [
    'assets/video11.png',
    'assets/video12.png',
    'assets/video13.png',
    'assets/video14.png',
    'assets/img_video_2.png'
  ];

  @override
  void initState() {
    super.initState();
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersive);
    currentPageIndex = initVideoIndex;
    _pageController = PageController(
      initialPage: initVideoIndex,
    );
    int totalLength = videoPaths.length;
    for (int index = 0; index < totalLength; index++) {
      var videoParams = VideoParams(
          videoPath: videoPaths[index],
          videoFirstFramePath: videoFirstFramePaths[index],
          type: VideoParams.TYPE_VIDEO);
      videoDatas.add(videoParams);
    }
    for (int index = 0; index < totalAdViewSize; index++) {
      var videoParams = VideoParams(
          videoPath: null,
          videoFirstFramePath: null,
          type: VideoParams.TYPE_AD);
      videoDatas.insert(index * 2 + 2, videoParams);
    }
    _initVideoPlayerState(initVideoIndex);
  }

  _initVideoPlayerState(int index) {
    videoPause = false;
    initVideoIndex = index;
    final controller =
        VideoPlayerController.asset(videoDatas[index].videoPath!);
    final old = _controller;

    old?.pause();
    debugPrint("video contoller paused.");
    _controller = controller;
    _videoPlayer = VideoPlayer(_controller!);

    debugPrint("video contoller changed.");
    setState(() {});

    controller
      ..initialize().then((_) {
        debugPrint("video controller initialized");
        old?.dispose();
        controller.play();
        setState(() {});
      });

    controller.setLooping(true);
  }

  void videoPlayerTaped() {
    bool videoPlaying = _controller!.value.isPlaying;
    if (videoPlaying) {
      _controller!.pause();
    } else {
      _controller!.play();
    }
    print("videoPlayerTaped videoPlaying $videoPlaying");
    setState(() {
      videoPause = videoPlaying;
    });
  }

  void _pageChanged(int index) {
    print("_pageChanged index is $index");
    currentPageIndex = index;
    var videoData = videoDatas[index];
    if (videoData.type == VideoParams.TYPE_AD) {
      print("_pageChanged ad view return");
      _controller!.pause();
      if (!cacheAd) {
        setState(() {
          viewRenderStatusMap['${index}'] = false;
        });
      }
      return;
    }
    _initVideoPlayerState(index);
  }

  void adViewCallBack(YFAdsNativeDrawFeedVideoCallBack reaperAdCallBack) {
    print(
        "adViewCallBack CallBack is ${reaperAdCallBack.method} videId is ${reaperAdCallBack.viewId}");
    if (YFAdsCallBackMethod.METHOD_DRAW_FEED_VIDEO_SUCCESS ==
        reaperAdCallBack.method) {
      String viewId = reaperAdCallBack.viewId!;
      if (!cacheAd) {
        setState(() {
          viewRenderStatusMap[viewId] = true;
        });
        return;
      }
      bool? cached = viewRenderStatusMap[viewId];
      print("adViewCallBack cached $cached");
      if (cached == null || !cached) {
        setState(() {
          viewRenderStatusMap[viewId] = true;
        });
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    var pauseView = videoPause
        ? Container(
            width: 48,
            height: 48,
            child: Icon(Icons.play_arrow),
            decoration: BoxDecoration(
                color: Colors.blue,
                borderRadius: BorderRadius.all(Radius.circular(24))),
          )
        : Container();

    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;

    var curVideoData = videoDatas[initVideoIndex];
    var overlayPictureWidget = curVideoData.type == VideoParams.TYPE_VIDEO
        ? Container(
            child: Image.asset(
            videoDatas[initVideoIndex].videoFirstFramePath!,
            fit: BoxFit.cover,
            height: double.infinity,
            width: double.infinity,
            alignment: Alignment.center,
          ))
        : Container(
            width: 0,
            height: 0,
          );

    var videoChildView = Center(
      child: _controller!.value.isInitialized
          ? AspectRatio(
              aspectRatio: _controller!.value.aspectRatio,
              child: GestureDetector(
                child: Stack(
                  alignment: AlignmentDirectional.center,
                  children: [_videoPlayer!, pauseView],
                ),
                onTap: videoPlayerTaped,
              ))
          : overlayPictureWidget,
    );

    var adProgressView = Center(
      child: Container(
        width: double.infinity,
        height: double.infinity,
        color: Colors.black12,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            CircularProgressIndicator(),
            Text(
              "加载中",
              textAlign: TextAlign.center,
              style: TextStyle(
                fontSize: 8,
                color: Colors.grey,
                decoration: TextDecoration.none,
              ),
            )
          ],
        ),
      ),
    );

    var pageView = PageView.builder(
      scrollDirection: Axis.vertical,
      itemBuilder: (BuildContext context, int index) {
        var videoData = videoDatas[index];
        if (videoData.type == VideoParams.TYPE_AD) {
          bool? viewRendered = viewRenderStatusMap[index];
          return Stack(
            alignment: AlignmentDirectional.center,
            children: [
              YFAdsNativeDrawFeedVideoView(
                backgroundColor: Colors.white70,
                cache: cacheAd,
                interruptVideoPlay: true,
                uniqueId: index,
                positionId: "a2c9d554-17d5-4a50-a5b6-c955eb5e3855",
                width: width,
                height: height,
                onAdCallBack: adViewCallBack,
              ),
              viewRendered == null || !viewRendered
                  ? adProgressView
                  : Container(
                      width: 0,
                      height: 0,
                    )
            ],
          );
        }
        return videoChildView;
      },
      onPageChanged: _pageChanged,
      controller: _pageController,
      itemCount: videoDatas.length,
    );

    return pageView;
  }

  @override
  void dispose() {
    super.dispose();
    _controller!.dispose();
    YFAds.instance.clearNativeDrawFeedVideoPageCache();
  }
}
